//---------------------------------------------------------------------------
// main.c : graphical user interface for rnx2rtkp
//
//          Copyright (C) 2007 by T.TAKASU, All rights reserved.
//
// version : $Revision: 3 $ $Date: 07/05/25 3:28 $
// history : 2007/03/20  1.0 new
//---------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <vcl.h>

#pragma hdrstop

#include "main.h"
#include "setdlg.h"
#include "convdlg.h"
#include "refdlg.h"
#include "viewer.h"
#include "plot.h"
#include "rtklib.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;

#define GOOGLE_EARTH "C:\\Program Files\\Google\\Google Earth\\googleearth.exe"

//---------------------------------------------------------------------------
extern "C" {
void printmsg(char *format, ...)
{
    va_list arg;
    char buff[1024];
    va_start(arg,format); vsprintf(buff,format,arg); va_end(arg);
    MainForm->Message->Caption=buff;
    MainForm->Message->Font->Color=strstr(buff,"error")?clRed:clBlue;
    MainForm->Update();
}
int printprog(int prog)
{
    Application->ProcessMessages();
    if (!MainForm->BtnExec->Enabled) return 1;
    MainForm->Progress->Position=prog;
    return 0;
}
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
	traceopen("trace.txt"); // for debug
	Datum=DynamicModel=IonoEst=TropEst=AntPcvCorr=AmbRes=0;
	OutCntResetBias=5; LockCntFixAmb=5;
	MaxAgeDiff=30.0; RejectThres=30.0;
	MeasErr1=100.0; MeasErr2=0.004; MeasErr3=0.003; MeasErr4=0.01;
	SatClkStab=1E-11; ValidThresAR=3.0;
	RovAnt=RefAnt=AntPcvFile=GoogleEarthFile="";
	RovAntE=RovAntN=RovAntU=RefAntE=RefAntN=RefAntU=0.0;
	RefList=new TStringList;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
	TIniFile *ini=new TIniFile("rnx2rtkp.ini");
	TimeStart->Checked =ini->ReadInteger("set","timestart",0);
	TimeEnd  ->Checked =ini->ReadInteger("set","timeend",0);
	TimeIntF ->Checked =ini->ReadInteger("set","timeintf",0);
	TimeY1->Text       =ini->ReadString ("set","timey1","2000/01/01");
	TimeH1->Text       =ini->ReadString ("set","timeh1","00:00:00");
	TimeY2->Text       =ini->ReadString ("set","timey2","2000/01/01");
	TimeH2->Text       =ini->ReadString ("set","timeh2","00:00:00");
	TimeInt->Text      =ini->ReadString ("set","timeint","0");
    PosMode            =ini->ReadInteger("set","posmode",2);
    Freq               =ini->ReadInteger("set","freq",1);
    Solution           =ini->ReadInteger("set","solution",0);
    ElMask             =ini->ReadFloat  ("set","elmask",10.0);
	RefPos->Checked    =ini->ReadInteger("set","refpos",0);
	RefType=ini->ReadInteger("set","reftype",0);
	RefType1->Checked  =!RefType;
	RefType2->Checked  =RefType;
	RefPos1->Text      =ini->ReadString ("set","refpos1","0");
    RefPos2->Text      =ini->ReadString ("set","refpos2","0");
    RefPos3->Text      =ini->ReadString ("set","refpos3","0");
    InputFile1->Text   =ini->ReadString ("set","inputfile1","");
    InputFile2->Text   =ini->ReadString ("set","inputfile2","");
    InputFile3->Text   =ini->ReadString ("set","inputfile3","");
    SolFormat          =ini->ReadInteger("set","solformat",0);
    TimeFormat         =ini->ReadInteger("set","timeformat",0);
    LatLonFormat       =ini->ReadInteger("set","latlonformat",0);
    FieldSep           =ini->ReadString ("set","fieldsep","");
    OutputFile->Text   =ini->ReadString ("set","outputfile","");
    InputFile1->Items  =ReadList(ini,"hist","inputfile1");
    InputFile2->Items  =ReadList(ini,"hist","inputfile2");
    InputFile3->Items  =ReadList(ini,"hist","inputfile3");
    OutputFile->Items  =ReadList(ini,"hist","outputfile");
    RefList            =ReadList(ini,"refpos","list");
    Datum              =ini->ReadInteger("set","datum",0);
    DynamicModel       =ini->ReadInteger("set","dynamicmodel",0);
    IonoEst            =ini->ReadInteger("set","ionoest",0);
    TropEst            =ini->ReadInteger("set","tropest",0);
	ValidThresAR       =ini->ReadFloat  ("set","validthresar",3.0);
	AmbRes             =ini->ReadInteger("set","ambres",0);
    OutCntResetBias    =ini->ReadInteger("set","outcntresetbias",5);
    LockCntFixAmb      =ini->ReadInteger("set","lockcntfixamb",5);
    MaxAgeDiff         =ini->ReadFloat  ("set","maxagediff",30.0);
    RejectThres        =ini->ReadFloat  ("set","rejectthres",30.0);
    MeasErr1           =ini->ReadFloat  ("set","measerr1",100.0);
    MeasErr2           =ini->ReadFloat  ("set","measerr2",0.004);
    MeasErr3           =ini->ReadFloat  ("set","measerr3",0.003);
    MeasErr4           =ini->ReadFloat  ("set","measerr4",0.01);
    SatClkStab         =ini->ReadFloat  ("set","satclkstab",1E-11);
    PrNoise1           =ini->ReadFloat  ("set","prnoise1",0.0);
    PrNoise2           =ini->ReadFloat  ("set","prnoise2",0.0);
    PrNoise3           =ini->ReadFloat  ("set","prnoise3",0.0);
    AntPcvCorr         =ini->ReadInteger("set","antpcvcorr",0);
    RovAnt             =ini->ReadString ("set","rovant","");
    RefAnt             =ini->ReadString ("set","refant","");
    RovAntE            =ini->ReadFloat  ("set","rovante",0.0);
    RovAntN            =ini->ReadFloat  ("set","rovantn",0.0);
    RovAntU            =ini->ReadFloat  ("set","rovantu",0.0);
    RefAntE            =ini->ReadFloat  ("set","refante",0.0);
    RefAntN            =ini->ReadFloat  ("set","refantn",0.0);
    RefAntU            =ini->ReadFloat  ("set","refantu",0.0);
    AntPcvFile         =ini->ReadString ("set","antpcvfile","");
    IntpRefObs         =ini->ReadInteger("set","intprefobs",0);
    NetRSCorr          =ini->ReadInteger("set","netrscorr",0);
    SatClkCorr         =ini->ReadInteger("set","satclkcorr",0);
    NetRSCorrFile      =ini->ReadString ("set","netrscorrfile","");
    SatClkCorrFile     =ini->ReadString ("set","satclkcorrfile","");
    GoogleEarthFile    =ini->ReadString ("set","googleearthfile",GOOGLE_EARTH);
    ConvDialog->StartTime->Checked   =ini->ReadInteger("conv","starttime",0);
    ConvDialog->TimeY1->Text         =ini->ReadString ("conv","timey1","2000/01/01");
    ConvDialog->TimeH1->Text         =ini->ReadString ("conv","timeh1","00:00:00");
    ConvDialog->TimeY2->Text         =ini->ReadString ("conv","timey2","2000/01/01");
    ConvDialog->TimeH2->Text         =ini->ReadString ("conv","timeh2","00:00:00");
    ConvDialog->TrackColor->ItemIndex=ini->ReadInteger("conv","trackcolor",5);
    ConvDialog->PointColor->ItemIndex=ini->ReadInteger("conv","pointcolor",5);
    ConvDialog->OutputAlt->ItemIndex =ini->ReadInteger("conv","outputalt",0);
    ConvDialog->OutputTime->ItemIndex=ini->ReadInteger("conv","outputtime",0);
    ConvDialog->AddOffset->Checked   =ini->ReadInteger("conv","addoffset",0);
    ConvDialog->Offset1->Text        =ini->ReadString ("conv","offset1","0");
    ConvDialog->Offset2->Text        =ini->ReadString ("conv","offset2","0");
    ConvDialog->Offset3->Text        =ini->ReadString ("conv","offset3","0");
    TSolPlot::PlotTypeD=ini->ReadInteger("plot","plottype",0);
    TSolPlot::QFlagD   =ini->ReadInteger("plot","qflag",0);
    TSolPlot::CColor[0]=(TColor)ini->ReadInteger("plot","color1",(int)clWhite);
    TSolPlot::CColor[1]=(TColor)ini->ReadInteger("plot","color2",(int)clGray);
    TSolPlot::CColor[2]=(TColor)ini->ReadInteger("plot","color3",(int)clBlack);
    TSolPlot::CColor[3]=(TColor)ini->ReadInteger("plot","color4",(int)clBlue);
    TSolPlot::PlotStyle=ini->ReadInteger("plot","plotstyle",0);
    TSolPlot::MarkSize =ini->ReadInteger("plot","marksize",5);
    TSolPlot::Origin   =ini->ReadInteger("plot","orgin",0);
    TSolPlot::AutoScale=ini->ReadInteger("plot","autoscale",0);
    TSolPlot::ShowArrow=ini->ReadInteger("plot","showarrow",0);
    TSolPlot::ShowStats=ini->ReadInteger("plot","showstats",0);
    TSolPlot::YRange   =ini->ReadFloat  ("plot","yrange",0);
    TSolPlot::TimeLabel=ini->ReadInteger("plot","timelabel",0);
    TSolPlot::OOPos[0] =ini->ReadFloat  ("plot","oopos1",0);
    TSolPlot::OOPos[1] =ini->ReadFloat  ("plot","oopos2",0);
    TSolPlot::OOPos[2] =ini->ReadFloat  ("plot","oopos3",0);
    delete ini;
    UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    TIniFile *ini=new TIniFile("rnx2rtkp.ini");
	ini->WriteInteger("set","timestart",TimeStart->Checked?1:0);
	ini->WriteInteger("set","timeend",TimeEnd->Checked?1:0);
	ini->WriteInteger("set","timeintf",TimeIntF->Checked?1:0);
	ini->WriteString ("set","timey1",TimeY1->Text);
	ini->WriteString ("set","timeh1",TimeH1->Text);
	ini->WriteString ("set","timey2",TimeY2->Text);
	ini->WriteString ("set","timeh2",TimeH2->Text);
	ini->WriteString ("set","timeint",TimeInt->Text);
    ini->WriteInteger("set","posmode",PosMode);
    ini->WriteInteger("set","freq",Freq);
    ini->WriteInteger("set","solution",Solution);
    ini->WriteFloat  ("set","elmask",ElMask);
    ini->WriteInteger("set","refpos",RefPos->Checked);
    ini->WriteInteger("set","reftype",RefType);
    ini->WriteString ("set","refpos1",RefPos1->Text);
    ini->WriteString ("set","refpos2",RefPos2->Text);
    ini->WriteString ("set","refpos3",RefPos3->Text);
    ini->WriteString ("set","inputfile1",InputFile1->Text);
    ini->WriteString ("set","inputfile2",InputFile2->Text);
    ini->WriteString ("set","inputfile3",InputFile3->Text);
    ini->WriteInteger("set","solformat",SolFormat);
    ini->WriteInteger("set","timeformat",TimeFormat);
    ini->WriteInteger("set","latlonformat",LatLonFormat);
    ini->WriteString ("set","fieldsep",FieldSep);
    ini->WriteString ("set","outputfile",OutputFile->Text);
    WriteList(ini,"hist","inputfile1",InputFile1->Items);
    WriteList(ini,"hist","inputfile2",InputFile2->Items);
    WriteList(ini,"hist","inputfile3",InputFile3->Items);
    WriteList(ini,"hist","outputfile",OutputFile->Items);
    WriteList(ini,"refpos","list",RefList);
    ini->WriteInteger("set","dynamicmodel",DynamicModel);
    ini->WriteInteger("set","ionoest",IonoEst);
    ini->WriteInteger("set","tropest",TropEst);
    ini->WriteFloat  ("set","validthresar",ValidThresAR);
    ini->WriteInteger("set","ambres",AmbRes);
    ini->WriteInteger("set","outcntresetbias",OutCntResetBias);
    ini->WriteInteger("set","lockcntfixamb",LockCntFixAmb);
    ini->WriteFloat  ("set","maxagediff",MaxAgeDiff);
    ini->WriteFloat  ("set","rejectthres",RejectThres);
    ini->WriteFloat  ("set","measerr1",MeasErr1);
    ini->WriteFloat  ("set","measerr2",MeasErr2);
    ini->WriteFloat  ("set","measerr3",MeasErr3);
    ini->WriteFloat  ("set","measerr4",MeasErr4);
    ini->WriteFloat  ("set","satclkstab",SatClkStab);
    ini->WriteFloat  ("set","prnoise1",PrNoise1);
    ini->WriteFloat  ("set","prnoise2",PrNoise2);
    ini->WriteFloat  ("set","prnoise3",PrNoise3);
    ini->WriteInteger("set","antpcvcorr",AntPcvCorr);
    ini->WriteString ("set","rovant",RovAnt);
    ini->WriteString ("set","refant",RefAnt);
    ini->WriteFloat  ("set","rovante",RovAntE);
    ini->WriteFloat  ("set","rovantn",RovAntN);
    ini->WriteFloat  ("set","rovantu",RovAntU);
    ini->WriteFloat  ("set","refante",RefAntE);
    ini->WriteFloat  ("set","refantn",RefAntN);
    ini->WriteFloat  ("set","refantu",RefAntU);
    ini->WriteString ("set","antpcvfile",AntPcvFile);
    ini->WriteInteger("set","intprefobs",IntpRefObs);
    ini->WriteInteger("set","netrscorr",NetRSCorr);
    ini->WriteInteger("set","satclkcorr",SatClkCorr);
    ini->WriteString ("set","netrscorrfile",NetRSCorrFile);
    ini->WriteString ("set","satclkcorrfile",SatClkCorrFile);
    ini->WriteString ("set","googleearthfile",GoogleEarthFile);
    ini->WriteInteger("conv","starttime",ConvDialog->StartTime->Checked);
    ini->WriteString ("conv","timey1",ConvDialog->TimeY1->Text);
    ini->WriteString ("conv","timeh1",ConvDialog->TimeH1->Text);
    ini->WriteString ("conv","timey2",ConvDialog->TimeY2->Text);
    ini->WriteString ("conv","timeh2",ConvDialog->TimeH2->Text);
    ini->WriteInteger("conv","trackcolor",ConvDialog->TrackColor->ItemIndex);
    ini->WriteInteger("conv","pointcolor",ConvDialog->PointColor->ItemIndex);
    ini->WriteInteger("conv","outputalt",ConvDialog->OutputAlt->ItemIndex);
    ini->WriteInteger("conv","outputtime",ConvDialog->OutputTime->ItemIndex);
    ini->WriteInteger("conv","addoffset",ConvDialog->AddOffset->Checked);
    ini->WriteString ("conv","offset1",ConvDialog->Offset1->Text);
    ini->WriteString ("conv","offset2",ConvDialog->Offset2->Text);
    ini->WriteString ("conv","offset3",ConvDialog->Offset3->Text);
    ini->WriteInteger("plot","plottype",TSolPlot::PlotTypeD);
    ini->WriteInteger("plot","qflag",TSolPlot::QFlagD);
    ini->WriteInteger("plot","color1",(int)TSolPlot::CColor[0]);
    ini->WriteInteger("plot","color2",(int)TSolPlot::CColor[1]);
    ini->WriteInteger("plot","color3",(int)TSolPlot::CColor[2]);
    ini->WriteInteger("plot","color4",(int)TSolPlot::CColor[3]);
    ini->WriteInteger("plot","plotstyle",TSolPlot::PlotStyle);
    ini->WriteInteger("plot","marksize",TSolPlot::MarkSize);
    ini->WriteInteger("plot","orgin",TSolPlot::Origin);
    ini->WriteInteger("plot","autoscale",TSolPlot::AutoScale);
    ini->WriteInteger("plot","showarrow",TSolPlot::ShowArrow);
    ini->WriteInteger("plot","showstats",TSolPlot::ShowStats);
    ini->WriteFloat  ("plot","yrange",TSolPlot::YRange);
    ini->WriteInteger("plot","timelabel",TSolPlot::TimeLabel);
    ini->WriteFloat  ("plot","oopos1",TSolPlot::OOPos[0]);
    ini->WriteFloat  ("plot","oopos2",TSolPlot::OOPos[1]);
    ini->WriteFloat  ("plot","oopos3",TSolPlot::OOPos[2]);
    delete ini;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnPlotClick(TObject *Sender)
{
	TSolPlot *plot=new TSolPlot(Application);
	plot->Show();
	plot->Read(OutputFile->Text);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnViewClick(TObject *Sender)
{
	TTextViewer *viewer=new TTextViewer(Application);
	viewer->Show();
	viewer->Read(OutputFile->Text);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnToKMLClick(TObject *Sender)
{
	ConvDialog->ShowModal();	
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnOptionClick(TObject *Sender)
{
	if (SetDialog->ShowModal()!=mrOk) return;
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnStopClick(TObject *Sender)
{
	printmsg("abort");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnExecClick(TObject *Sender)
{
	int nfile=0;
	char s[3][256]={{0}};
	char *infile[]={s[0],s[1],s[2]},outfile[256],sep[32]=" ";
	char corrfile[256]="";
	
	if (BtnExec->Caption=="Abort") {
		BtnExec->Enabled=false;
		return;
	}
	if (InputFile1->Text=="") {
		printmsg("error : no rover obs file");
		return;
	}
	if (InputFile3->Text==""&&PosMode>0) {
		printmsg("error : no ref obs file");
		return;
	}
	if (OutputFile->Text=="") {
		printmsg("error : no output file");
		return;
	}
	strcpy(infile[nfile++],InputFile1->Text.c_str());
	if (PosMode) {
		strcpy(infile[nfile++],InputFile3->Text.c_str());
	}
	if (InputFile2->Text!="") {
		strcpy(infile[nfile++],InputFile2->Text.c_str());
	}
	else {
		strcpy(infile[nfile++],InputFile1->Text.c_str());
		infile[nfile-1][strlen(infile[nfile-1])-1]='n';
	}
	if (SatClkCorr) {
		strcpy(corrfile,SatClkCorrFile.c_str());
	}
	gtime_t ts,te;
	double es[6]={1970,1,1},ee[6]={2038,1,1},tint=0.0;
	if (TimeStart->Checked) {
		sscanf(TimeY1->Text.c_str(),"%lf/%lf/%lf",es  ,es+1,es+2);
		sscanf(TimeH1->Text.c_str(),"%lf:%lf:%lf",es+3,es+4,es+5);
	}
	if (TimeEnd->Checked) {
		sscanf(TimeY2->Text.c_str(),"%lf/%lf/%lf",ee  ,ee+1,ee+2);
		sscanf(TimeH2->Text.c_str(),"%lf:%lf:%lf",ee+3,ee+4,ee+5);
	}
	ts=epoch2time(es);
	te=epoch2time(ee);
	if (TimeIntF->Checked) tint=TimeInt->Text.ToDouble();
	opt_t opt;
	if (!GetOption(opt)) return;
	int dirs=opt.mode==0||opt.mode==3?0:Solution;
	int times=TimeFormat>1;
	int timef=TimeFormat;
	int timeu=3;
	int posf=SolFormat;
	int degf=LatLonFormat;
	if (FieldSep!="") strcpy(sep,FieldSep.c_str());
	printmsg("");
	FILE *fp;
	AnsiString msg="File exists. Overwrite it?";
	if ((fp=fopen(OutputFile->Text.c_str(),"r"))) {
		if (MessageDlg(msg,mtConfirmation,mbOKCancel,0)!=mrOk) return;
		fclose(fp);
	}
	strcpy(outfile,OutputFile->Text.c_str());
	BtnExec->Caption="Abort";
	BtnExit->Enabled=false;
	BtnView->Enabled=false;
	BtnToKML->Enabled=false;
	BtnPlot->Enabled=false;
	BtnOption->Enabled=false;
	printmsg("processing...");
	if (!postpos(ts,te,tint,&opt,sep,timeu,dirs,times,timef,posf,degf,infile,
	             nfile,outfile,corrfile)) {
		AddHist(InputFile1);
		AddHist(InputFile2);
		AddHist(InputFile3);
		AddHist(OutputFile);
	}
	if (Message->Caption=="processing...") printmsg("done");
	BtnExec->Caption="E&xecute";
	BtnExec->Enabled=true;
	BtnExit->Enabled=true;
	BtnView->Enabled=true;
	BtnToKML->Enabled=true;
	BtnPlot->Enabled=true;
	BtnOption->Enabled=true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnExitClick(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeStartClick(TObject *Sender)
{
	UpdateEnable();	
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PosModeChange(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SolutionChange(TObject *Sender)
{
	UpdateEnable();	
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefPosClick(TObject *Sender)
{
	UpdateEnable();	
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefType1Click(TObject *Sender)
{
	AnsiString s;
	double pos[3],rb[3];
	if (!RefType) return; else RefType=0;
	rb[0]=RefPos1->Text.ToDouble();
	rb[1]=RefPos2->Text.ToDouble();
	rb[2]=RefPos3->Text.ToDouble();
	ecef2pos(rb,pos);
	RefPos1->Text=s.sprintf("%.9f",pos[0]*R2D);
	RefPos2->Text=s.sprintf("%.9f",pos[1]*R2D);
	RefPos3->Text=s.sprintf("%.4f",pos[2]);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefType2Click(TObject *Sender)
{
	AnsiString s;
	double pos[3],rb[3];
	if (RefType) return; else RefType=1;
	pos[0]=RefPos1->Text.ToDouble()*D2R;
	pos[1]=RefPos2->Text.ToDouble()*D2R;
	pos[2]=RefPos3->Text.ToDouble();
	pos2ecef(pos,rb);
	RefPos1->Text=s.sprintf("%.4f",rb[0]);
	RefPos2->Text=s.sprintf("%.4f",rb[1]);
	RefPos3->Text=s.sprintf("%.4f",rb[2]);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnRefPosClick(TObject *Sender)
{
	AnsiString s;
	double pos[3],p[3];
	pos[0]=RefPos1->Text.ToDouble();
	pos[1]=RefPos2->Text.ToDouble();
	pos[2]=RefPos3->Text.ToDouble();
	if (RefType2->Checked) {
		ecef2pos(pos,p);
		p[0]*=R2D; p[1]*=R2D;
		for (int i=0;i<3;i++) pos[i]=p[i];
	}
	for (int i=0;i<3;i++) RefDialog->Pos[i]=pos[i];
	if (RefDialog->ShowModal()!=mrOk) return;
	for (int i=0;i<3;i++) pos[i]=RefDialog->Pos[i];
	if (RefType2->Checked) {
		pos[0]*=D2R; pos[1]*=D2R;
		pos2ecef(pos,p);
		for (int i=0;i<3;i++) pos[i]=p[i];
	}
	RefPos1->Text=s.sprintf("%.9f",pos[0]);
	RefPos2->Text=s.sprintf("%.9f",pos[1]);
	RefPos3->Text=s.sprintf("%.4f",pos[2]);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputFile1Click(TObject *Sender)
{
    OpenDialog->FileName=InputFile1->Text;
    OpenDialog->FilterIndex=1;
    if (!OpenDialog->Execute()) return;
    InputFile1->Text=OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputFile2Click(TObject *Sender)
{
    OpenDialog->FileName=InputFile2->Text;
    OpenDialog->FilterIndex=2;
    if (!OpenDialog->Execute()) return;
    InputFile2->Text=OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputFile3Click(TObject *Sender)
{
    OpenDialog->FileName=InputFile3->Text;
    OpenDialog->FilterIndex=1;
    if (!OpenDialog->Execute()) return;
    InputFile3->Text=OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnOutputFileClick(TObject *Sender)
{
    SaveDialog->FileName=OutputFile->Text;
    if (!SaveDialog->Execute()) return;
    OutputFile->Text=SaveDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputViiew1Click(TObject *Sender)
{
	if (InputFile1->Text=="") return;
	TTextViewer *viewer=new TTextViewer(Application);
	viewer->Show();
	viewer->Read(InputFile1->Text);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputView2Click(TObject *Sender)
{
	if (InputFile2->Text=="") return;
	TTextViewer *viewer=new TTextViewer(Application);
	viewer->Show();
	viewer->Read(InputFile2->Text);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputView3Click(TObject *Sender)
{
	if (InputFile3->Text=="") return;
	TTextViewer *viewer=new TTextViewer(Application);
	viewer->Show();
	viewer->Read(InputFile3->Text);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SolFormatChange(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateEnable(void)
{

	RefPos       ->Enabled=PosMode>=1;
	RefType1     ->Enabled=PosMode>=1&&RefPos->Checked;
	RefType2     ->Enabled=PosMode>=1&&RefPos->Checked;
	RefPos1      ->Enabled=PosMode>=1&&RefPos->Checked;
	RefPos2      ->Enabled=PosMode>=1&&RefPos->Checked;
	RefPos3      ->Enabled=PosMode>=1&&RefPos->Checked;
	BtnRefPos    ->Enabled=PosMode>=1&&RefPos->Checked;
	InputFile3   ->Enabled=PosMode>=1;
	BtnInputFile3->Enabled=PosMode>=1;
	BtnInputView3->Enabled=PosMode>=1;
	TimeY1       ->Enabled=TimeStart->Checked;
	TimeH1       ->Enabled=TimeStart->Checked;
	TimeY2       ->Enabled=TimeEnd->Checked;
	TimeH2       ->Enabled=TimeEnd->Checked;
	TimeInt      ->Enabled=TimeIntF->Checked;
	LabelTimeInt ->Enabled=TimeIntF->Checked;
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::GetOption(opt_t &opt)
{
	opt=opt_default;
	opt.mode=PosMode;
	opt.nf=Freq+1;
	GetRefPos(opt.rb);
	opt.elmin=ElMask*D2R;
	opt.modear=PosMode>=2&&AmbRes==2&&Solution<=1;
	opt.maxout=OutCntResetBias;
	opt.minlock=LockCntFixAmb;
	opt.estiono=IonoEst;
	opt.esttrop=TropEst;
	opt.err[0]=MeasErr1;
	opt.err[1]=MeasErr2;
	opt.err[2]=MeasErr3;
	opt.err[3]=MeasErr4;
	opt.sclkstab=SatClkStab;
	opt.prn[0]=PrNoise1;
	opt.prn[1]=PrNoise2;
	opt.prn[2]=PrNoise3;
	if (AntPcvCorr) {
		if (readpcv(AntPcvFile.c_str(),RovAnt.c_str(),&opt.pcvu)) {
			printmsg("error : no antenna pcv : %s",RovAnt.c_str());
			return 0;
		}
		if (readpcv(AntPcvFile.c_str(),RefAnt.c_str(),&opt.pcvr)) {
			printmsg("error : no antenna pcv : %s",RefAnt.c_str());
			return 0;
		}
		opt.pcvu.ecc[0]=RovAntE;
		opt.pcvu.ecc[1]=RovAntN;
		opt.pcvu.ecc[2]=RovAntU;
		opt.pcvr.ecc[0]=RefAntE;
		opt.pcvr.ecc[1]=RefAntN;
		opt.pcvr.ecc[2]=RefAntU;
	}
	opt.thresar=AmbRes==0?0.0:ValidThresAR;
	opt.maxtdiff=MaxAgeDiff;
	opt.maxinno=RejectThres;
	opt.intpref=IntpRefObs;
	return 1;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetRefPos(double *rb)
{
	double pos[3];
	if (!RefPos->Checked) return;
	if (!RefType) {
		pos[0]=RefPos1->Text.ToDouble()*D2R;
		pos[1]=RefPos2->Text.ToDouble()*D2R;
		pos[2]=RefPos3->Text.ToDouble();
		pos2ecef(pos,rb);
	}
	else {
		rb[0]=RefPos1->Text.ToDouble();
		rb[1]=RefPos2->Text.ToDouble();
		rb[2]=RefPos3->Text.ToDouble();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddHist(TComboBox *combo)
{
    AnsiString hist=combo->Text;
    if (hist=="") return;
    TStrings *list=combo->Items;
    int i=list->IndexOf(hist);
    if (i>=0) list->Delete(i);
    list->Insert(0,hist);
    for (int i=list->Count-1;i>=10;i--) list->Delete(i);
    combo->ItemIndex=0;
}
//---------------------------------------------------------------------------
TStringList * __fastcall TMainForm::ReadList(TIniFile *ini, AnsiString cat,
    AnsiString key)
{
    TStringList *list=new TStringList;
    for (int i=0;i<100;i++) {
        AnsiString s(i),item=ini->ReadString(cat,key+s,"");
        if (item!="") list->Add(item); else break;
    }
    return list;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WriteList(TIniFile *ini, AnsiString cat,
    AnsiString key, TStrings *list)
{
    for (int i=0;i<list->Count;i++) {
        AnsiString s(i);
        ini->WriteString(cat,key+s,list->Strings[i]);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeIntFClick(TObject *Sender)
{
	UpdateEnable();	
}
//---------------------------------------------------------------------------

